/* 
** Module   : EntryFieldGuiImpl.java
** Abstract : combo-box editable text field implementation, GUI-specific version.
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20150605 Created initial version.
*/

package com.goldencode.p2j.ui.client.gui;

// this usage of AWT events is intentional, they are being used separately from the
// AWT/Swing event model itself; no other AWT/Swing usage is occurring in this class

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.event.*;

/**
 * Implementation of Progress entry field inside for GUI combo-box.
 */
public class EntryFieldGuiImpl
extends FillInGuiImpl
{
   /** Reference to the possible combo-box object. */
   private ComboBoxGuiImpl comboBoxRef = null;

   /**
    * Special constructor used to restore component using given configuration.
    * <p>
    * Call {@link #initialize} to finish the widget initialization.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public EntryFieldGuiImpl(WidgetId id, FillInConfig cfg)
   {
      super(id, cfg);
   }

   /**
    * Special constructor used to restore component using given configuration.
    * <p>
    * Call {@link #initialize} to finish the widget initialization.
    * 
    * @param   cfg
    *          Configuration data.
    * @param   cbbRef
    *          Combo-box reference.
    */
   public EntryFieldGuiImpl(FillInConfig cfg, ComboBoxGuiImpl cbbRef)
   {
      this(cfg.id, cfg);
      comboBoxRef = cbbRef;
   }

   /**
    * Process GUI specific keys for combo-box entry field:
    * <ul>
    *    <li><code>SE_VALUE_CHANGED</code> - inform combo-box the value changed.</li>
    * </ul>
    * 
    * @param   ke
    *          Key event structure.
    */
   @Override
   public void processKeyEvent(KeyInput ke)
   {
      int key = ke.actionCode();

      // in combo-box mode is active and we have value changed in fill-in
      if (key == Keyboard.SE_VALUE_CHANGED)
      {
         // stop handling this event for fill-in
         ke.consume();
         // and reroute them to the enclosing combo-box
         comboBoxRef.entryFieldChanged(getText());
      }
      else
      {
         super.processKeyEvent(ke);
      }
   }

   /**
    * Get widget physical dimension, expressed in units native to UI type.
    *  
    * @return  widget dimension.
    */
   @Override
   public NativeDimension physicalDimension()
   {
      NativeDimension dim = super.physicalDimension();
      
      // if widget inside combo-box, need to decrease height
      dim.height -= ComboBoxGuiImpl.ENTRY_DELTA;
      
      return dim;
   }

   /**
    * Get widget width.
    * 
    * @return  Widget width.
    */
   @Override
   public double width()
   {
      double origWidth = super.width();
      
      // if widget inside combo-box, need to increase width
      origWidth += 4.0;
      
      return origWidth;
   }

   /**
    * Determine the portion of the widget's editable part.  This may be different than the widget's
    * width, as determined by the WIDTH-CHARS or WIDTH-PIXELS attribute.
    * <p>
    * For GUI widgets, this is the physical dimension without the insets.
    * 
    * @return   The editable dimension of this widget, in native units.
    */
   @Override
   protected NativeDimension editableNativeDimension()
   {
      NativeDimension dim = physicalDimension();

      // combo-box mode needs pure physical dimension based calculation
      dim.width -= 1;
      dim.height-= nativeInsets.top + nativeInsets.bottom;
   
      return dim;
   }

   /**
    * Draws the border box around the fill-in.
    * 
    * @param    borderColor
    *           The border box color.
    * @param    borderDim
    *           The border box size.
    */
   @Override
   protected void drawBorder(ColorRgb borderColor, NativeDimension borderDim)
   {
      // entry field in combo-box has no border
   }
}
